@@ -1,18 +1,18 @@
 ;;;-*- Mode: Lisp; Package: CCL -*-
 ;;;
-;;;   Copyright (C) 2005-2009 Clozure Associates and contributors.
-;;;   This file is part of Clozure CL.  
+;;; Copyright 2005-2009 Clozure Associates
 ;;;
-;;;   Clozure CL is licensed under the terms of the Lisp Lesser GNU Public
-;;;   License , known as the LLGPL and distributed with Clozure CL as the
-;;;   file "LICENSE".  The LLGPL consists of a preamble and the LGPL,
-;;;   which is distributed with Clozure CL as the file "LGPL".  Where these
-;;;   conflict, the preamble takes precedence.  
+;;; Licensed under the Apache License, Version 2.0 (the "License");
+;;; you may not use this file except in compliance with the License.
+;;; You may obtain a copy of the License at
 ;;;
-;;;   Clozure CL is referenced in the preamble as the "LIBRARY."
+;;;     http://www.apache.org/licenses/LICENSE-2.0
 ;;;
-;;;   The LLGPL is also available online at
-;;;   http://opensource.franz.com/preamble.html
+;;; Unless required by applicable law or agreed to in writing, software
+;;; distributed under the License is distributed on an "AS IS" BASIS,
+;;; WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+;;; See the License for the specific language governing permissions and
+;;; limitations under the License.
 
 (in-package "CCL")
 
@@ -86,6 +86,7 @@
     name
     frag
     offset
+    align
     )
 
   (defstruct (frag (:include ccl::dll-node)
@@ -579,7 +580,10 @@
                    (arch::target-t-offset  (backend-target-arch *target-backend*))))))
 
       (progn
-        (when (symbolp form)
+        (when (and (symbolp form) (find form (arch::target-subprims-table
+                                              (backend-target-arch *target-backend*))
+                                        :test #'string-equal 
+                                        :key #'subprimitive-info-name))
           (multiple-value-bind (offset condition)
               (ignore-errors (subprim-name->offset form))
             (unless condition  (setq form offset)))))
@@ -1351,44 +1355,31 @@
                     (:expr64 (emit-quad frag pos (x86-lap-expression-value arg)))
 		    (:self (emit-long frag pos (x86-lap-expression-value arg)))))))))))))
 
-(defstatic *x86-32-bit-lap-nops*
-  #(
-    #()
-    #(#x90)                             ; nop                  
-    #(#x89 #xf6)                        ; movl %esi,%esi       
-    #(#x8d #x76 #x00)                   ; leal 0(%esi),%esi    
-    #(#x8d #x74 #x26 #x00)              ; leal 0(%esi,1),%esi  
-    #(#x90 #x8d #x74 #x26 #x00)         ; nop ; leal 0(%esi,1),%esi  
-    #(#x8d #xb6 #x00 #x00 #x00 #x00)    ; leal 0L(%esi),%esi   
-    #(#x8d #xb4 #x26 #x00 #x00 #x00 #x00) ; leal 0L(%esi,1),%esi 
-  )
-  "Allegedly, many implementations recognize these instructions and
-execute them very quickly.")
+;;; Recommended multi-byte nop instructions.  See Intel 64 and IA-32
+;;; Architectures Software Development Manual, Volume 2B, entry for
+;;; NOP.
+(defstatic *x86-recommended-nops*
+  #(#()
+    #(#x90)
+    #(#x66 #x90)
+    #(#x0f #x1f #x00)
+    #(#x0f #x1f #x40 #x00)
+    #(#x0f #x1f #x44 #x00 #x00)
+    #(#x66 #x0f #x1f #x44 #x00 #x00)
+    #(#x0f #x1f #x80 #x00 #x00 #x00 #x00)
+    #(#x0f #x1f #x84 #x00 #x00 #x00 #x00 #x00)
+    ))
 
-(defstatic *x86-32-bit-lap-nops-8*
-  #(#x90 #x8d #xb4 #x26 #x00 #x00 #x00 #x00))
+(defstatic *x86-9-byte-nop* #(#x66 #x0f #x1f #x84 #x00 #x00 #x00 #x00 #x00))
 
 (defun frag-emit-nops (frag count)
-  (target-word-size-case
-   (32
-    (do* ((c count (- c 8)))
-         ((< c 8)
-          (let* ((v (svref *x86-32-bit-lap-nops* c)))
-            (dotimes (i c)
-              (frag-push-byte frag (svref v i)))))
-      (dotimes (i 8)
-        (frag-push-byte frag (svref *x86-32-bit-lap-nops-8* i)))))
-   (64
-    (let* ((nnops (ash (+ count 3) -2))
-           (len (floor count nnops))
-           (remains (- count (* nnops len))))
-      (dotimes (i remains)
-        (dotimes (k len) (frag-push-byte frag #x66))
-        (frag-push-byte frag #x90))
-      (do* ((i remains (1+ i)))
-           ((= i nnops))
-        (dotimes (k (1- len)) (frag-push-byte frag #x66))
-        (frag-push-byte frag #x90))))))
+  (do ((c count (- c 9)))
+      ((< c 9)
+       (let ((v (svref *x86-recommended-nops* c)))
+	 (dotimes (i c)
+	   (frag-push-byte frag (svref v i)))))
+    (dotimes (i 9)
+      (frag-push-byte frag (svref *x86-9-byte-nop* i)))))
   
 (defun fill-for-alignment (frag-list)
   (ccl::do-dll-nodes (frag frag-list)
@@ -1546,13 +1537,8 @@ execute them very quickly.")
     (x86-lap-directive frag-list :byte 0) ;regsave mask
     (emit-x86-lap-label frag-list entry-code-tag)
 
-    (let* ((first (car forms)))
-      (when (eq (car first) 'let)
-        (let* ((form0 (caddr first)))
-          (when (and (consp form0) (symbolp (car form0)))
-        
-            (unless (equalp (string (car form0)) "recover-fn-from-rip")
-              (x86-lap-form `(lea (@ (:^ ,entry-code-tag) (% rip)) (% fn)) frag-list instruction main-frag-list exception-frag-list))))))
+    (unless (equalp (car forms) '(recover-fn-from-rip))
+      (x86-lap-form `(lea (@ (:^ ,entry-code-tag) (% rip)) (% fn)) frag-list instruction main-frag-list exception-frag-list))
     (dolist (f forms)
       (setq frag-list (x86-lap-form f frag-list instruction main-frag-list exception-frag-list)))
     (setq frag-list main-frag-list)
