@@ -1,18 +1,18 @@
 ;;;-*-Mode: LISP; Package: ccl -*-
 ;;;
-;;;   Copyright (C) 2008-2009 Clozure Associates and contributors
-;;;   This file is part of Clozure CL.  
+;;; Copyright 2008-2009 Clozure Associates
 ;;;
-;;;   Clozure CL is licensed under the terms of the Lisp Lesser GNU Public
-;;;   License , known as the LLGPL and distributed with Clozure CL as the
-;;;   file "LICENSE".  The LLGPL consists of a preamble and the LGPL,
-;;;   which is distributed with Clozure CL as the file "LGPL".  Where these
-;;;   conflict, the preamble takes precedence.  
+;;; Licensed under the Apache License, Version 2.0 (the "License");
+;;; you may not use this file except in compliance with the License.
+;;; You may obtain a copy of the License at
 ;;;
-;;;   Clozure CL is referenced in the preamble as the "LIBRARY."
+;;;     http://www.apache.org/licenses/LICENSE-2.0
 ;;;
-;;;   The LLGPL is also available online at
-;;;   http://opensource.franz.com/preamble.html
+;;; Unless required by applicable law or agreed to in writing, software
+;;; distributed under the License is distributed on an "AS IS" BASIS,
+;;; WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+;;; See the License for the specific language governing permissions and
+;;; limitations under the License.
 
 ;;; Shared compiler backend utilities and infrastructure.
 
@@ -361,6 +361,7 @@
          seg vreg xfer (acode-operands acode)))
 
 
+
 (defun acode-constant-p (form)
   ;; This returns (values constant-value constantp); some code
   ;; may need to check constantp if constant-value is nil.
@@ -370,9 +371,19 @@
            (values nil t))
           ((eql op (%nx1-operator t))
            (values t t))
-          ((or (eql op (%nx1-operator fixnum))
-               (eql op (%nx1-operator immediate)))
+          ((eql op (%nx1-operator fixnum))
            (values (car (acode-operands form)) t))
+          ((eql op (%nx1-operator immediate))
+           ;; recognize the acode produced for LOAD-TIME-VALUE by
+           ;; COMPILE-FILE as something non-constant.
+           (if (and 
+                (consp (car (acode-operands form)))
+                *load-time-eval-token*
+                (eq (car (car (acode-operands form))) *load-time-eval-token*))
+             (values nil nil)
+                             
+                    
+             (values (car (acode-operands form)) t)))
           (t (values nil nil)))))
 
 (defun acode-constant-fold-binop (seg vreg xfer x y function)
@@ -510,9 +521,11 @@
         (or (var-type var)
             (setf (var-type var)
                   (let* ((initform (var-initform var)))
-                    (cond ((and initform (not (logbitp $vbitsetq bits)))
+                    (cond ((and initform
+                                (not (or (logbitp $vbitsetq bits)
+                                         (logbitp $vbitclosed bits))))
                            (acode-form-type initform trust-decls))
                           ((and trust-decls (var-declared-type var)))
                           (t '*))))))))
 
-            
\ No newline at end of file
+            
