@@ -1,19 +1,18 @@
 ;;; -*- Mode: Lisp; Package: CCL -*-
 ;;;
-;;;   Copyright (C) 2009 Clozure Associates
-;;;   Copyright (C) 1994-2001 Digitool, Inc
-;;;   This file is part of Clozure CL.  
+;;; Copyright 1994-2009 Clozure Associates
 ;;;
-;;;   Clozure CL is licensed under the terms of the Lisp Lesser GNU Public
-;;;   License , known as the LLGPL and distributed with Clozure CL as the
-;;;   file "LICENSE".  The LLGPL consists of a preamble and the LGPL,
-;;;   which is distributed with Clozure CL as the file "LGPL".  Where these
-;;;   conflict, the preamble takes precedence.  
+;;; Licensed under the Apache License, Version 2.0 (the "License");
+;;; you may not use this file except in compliance with the License.
+;;; You may obtain a copy of the License at
 ;;;
-;;;   Clozure CL is referenced in the preamble as the "LIBRARY."
+;;;     http://www.apache.org/licenses/LICENSE-2.0
 ;;;
-;;;   The LLGPL is also available online at
-;;;   http://opensource.franz.com/preamble.html
+;;; Unless required by applicable law or agreed to in writing, software
+;;; distributed under the License is distributed on an "AS IS" BASIS,
+;;; WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+;;; See the License for the specific language governing permissions and
+;;; limitations under the License.
 
 ;;; level-0;x86;x86-misc.lisp
 
@@ -407,6 +406,21 @@
   (shrq ($ 1) (% arg_z))
   (single-value-return))
 
+(defx86lapfunction rdtscp ()
+  (:byte #x0f)                          ;three-byte rdtscp opcode
+  (:byte #x01)                         ;is #x0f #x01 xf9
+  (:byte #xf9)
+  (shlq ($ 32) (% rdx))
+  (orq (% rdx) (% rax))
+  (imul ($ (* 2 target::node-size)) (% rax) (% arg_z))
+  (shrq ($ 1) (% arg_z))
+  (shlq ($ target::fixnumshift) (% rcx))
+  (push (% arg_z))
+  (push (% rcx))
+  (set-nargs 2)
+  (lea (@ '2 (% rsp)) (% temp0))
+  (jmp-subprim .SPvalues))
+
 ;;; Return all 64 bits of the time-stamp counter as an unsigned integer.
 (defx86lapfunction rdtsc64 ()
   (:byte #x0f)                          ;two-byte rdtsc opcode
@@ -788,14 +802,8 @@
   (lea (@ (- target::fulltag-misc target::node-size) (% imm0)) (% arg_z))
   (single-value-return))
 
-(defx86lapfunction get-saved-register-values ()
-  (movq (% rsp) (% temp0))
-  (push (% save0))
-  (push (% save1))
-  (push (% save2))
-  (push (% save3))                      ; this'd be the TCR on Win64.
-  (set-nargs 4)
-  (jmp-subprim .SPvalues))
+(defun get-saved-register-values ()
+  (values))
 
 
 (defx86lapfunction %current-db-link ()
@@ -939,132 +947,14 @@
 
 ;;; For win64 (which doesn't really have a "save3" register), the code
 ;;; which instantiates this should always set save3-offset to 0.
-(defx86lapfunction %%apply-in-frame-proto ()
-  (:fixed-constants (target-frame target-catch target-db-link target-xcf target-tsp target-foreign-sp save0-offset save1-offset save2-offset save3-offset function args))
-  (check-nargs 0)
-  ;;(uuo-error-debug-trap)
-  (movq (@ 'target-catch (% fn)) (% temp0))
-  (xorl (%l imm0) (%l imm0))
-  (cmpb ($ x8664::fulltag-nil) (%b temp0))
-  (movq (:rcontext target::tcr.catch-top) (% arg_z))
-  (jz @did-catch)
-  @find-catch
-  (testq (% arg_z) (% arg_z))
-  (jz @did-catch)                       ; never found target catch
-  (addq ($ '1)  (% imm0))
-  (cmpq (% temp0) (% arg_z))
-  (je @found-catch)
-  (movq (@ target::catch-frame.link (% arg_z)) (% arg_z))
-  (jmp @find-catch)
-  @found-catch
-  (set-nargs 0)                         ; redundant, but ...
-  (lea (@ (:^ @back-from-nthrow) (% fn)) (% ra0))
-  (:talign 4)
-  (jmp-subprim .SPnthrowvalues)
-  @back-from-nthrow
-  (recover-fn-from-rip)
-  @did-catch
-  ;; Restore special bindings
-  (movq (@ 'target-db-link (% fn)) (% imm0))
-  (cmpb ($ x8664::fulltag-nil) (%b imm0))
-  (jz @no-unbind)
-  (call-subprim .SPunbind-to)
-  @no-unbind
-  ;; If there's at least one exception frame between the target
-  ;; frame and the last catch (or the point of departure), restore
-  ;; the NVRs and foreign sp from the oldest such frame
-  (movq (@ 'target-xcf (% fn)) (% arg_z))
-  (cmpb ($ x8664::fulltag-nil) (%b arg_z))
-  (jz @no-xcf)
-  (movq (@ target::xcf.xp (% arg_z)) (% arg_y))
-  ;; arg_y points to a "portable" ucontext.  Find the platform-specifc
-  ;; "gpr vector" in the uc_mcontext, load the NVRs and stack/frame
-  ;; pointer from there.
-  #+linuxx8664-target
-  (progn
-    (addq ($ gp-regs-offset) (% arg_y))
-    (movq (@ (* #$REG_R15 8) (% arg_y)) (% r15))
-    (movq (@ (* #$REG_R14 8) (% arg_y)) (% r14))
-    (movq (@ (* #$REG_R12 8) (% arg_y)) (% r12))
-    (movq (@ (* #$REG_R11 8) (% arg_y)) (% r11))
-    (movq (@ (* #$REG_RBP 8) (% arg_y)) (% rbp))
-    (movq (@ (* #$REG_RSP 8) (% arg_y)) (% rsp)))
-  #+freebsdx8664-target
-  (progn
-    ;; If you think that this is ugly, just wait until you see the Darwin
-    ;; version.
-    (addq ($ gp-regs-offset) (% arg_y))
-    (movq (@ (ash (foreign-record-field-offset (%find-foreign-record-type-field (parse-foreign-type '(:struct :__mcontext)) :mc_r15)) -3) (% arg_y)) (% r15))
-    (movq (@ (ash (foreign-record-field-offset (%find-foreign-record-type-field (parse-foreign-type '(:struct :__mcontext)) :mc_r14)) -3) (% arg_y)) (% r14))
-    (movq (@ (ash (foreign-record-field-offset (%find-foreign-record-type-field (parse-foreign-type '(:struct :__mcontext)) :mc_r12)) -3) (% arg_y)) (% r12))
-    (movq (@ (ash (foreign-record-field-offset (%find-foreign-record-type-field (parse-foreign-type '(:struct :__mcontext)) :mc_r11)) -3) (% arg_y)) (% r11))
-    (movq (@ (ash (foreign-record-field-offset (%find-foreign-record-type-field (parse-foreign-type '(:struct :__mcontext)) :mc_rbp)) -3) (% arg_y)) (% rbp))
-    (movq (@ (ash (foreign-record-field-offset (%find-foreign-record-type-field (parse-foreign-type '(:struct :__mcontext)) :mc_rsp)) -3) (% arg_y)) (% rsp)))
-  #+darwinx8664-target
-  (progn
-    ;; Yes, this is ugly.
-    (movq (@ (ash (foreign-record-field-offset (%find-foreign-record-type-field (parse-foreign-type '(:struct :__darwin_ucontext)) :uc_mcontext)) -3) (% arg_y)) (% arg_y))
-    (addq (@ (ash (foreign-record-field-offset (%find-foreign-record-type-field (parse-foreign-type '(:struct :__darwin_mcontext64)) :__ss)) -3)) (% arg_y))
-    (movq (@ (ash (foreign-record-field-offset (%find-foreign-record-type-field (parse-foreign-type '(:struct :__darwin_x86_thread_state64)) :__r15)) -3) (% arg_y)) (% r15))
-    (movq (@ (ash (foreign-record-field-offset (%find-foreign-record-type-field (parse-foreign-type '(:struct :__darwin_x86_thread_state64)) :__r14)) -3) (% arg_y)) (% r14))
-    (movq (@ (ash (foreign-record-field-offset (%find-foreign-record-type-field (parse-foreign-type '(:struct :__darwin_x86_thread_state64)) :__r12)) -3) (% arg_y)) (% r12))
-    (movq (@ (ash (foreign-record-field-offset (%find-foreign-record-type-field (parse-foreign-type '(:struct :__darwin_x86_thread_state64)) :__r11)) -3) (% arg_y)) (% r11))
-    (movq (@ (ash (foreign-record-field-offset (%find-foreign-record-type-field (parse-foreign-type '(:struct :__darwin_x86_thread_state64)) :__rbp)) -3) (% arg_y)) (% rbp))
-    (movq (@ (ash (foreign-record-field-offset (%find-foreign-record-type-field (parse-foreign-type '(:struct :__darwin_x86_thread_state64)) :__rsp)) -3) (% arg_y)) (% rsp)))
-  ;; This is our best (possibly only) chance to get
-  ;; the foreign sp right.
-  (movq (@ target::xcf.prev-xframe (% arg_z)) (% temp0))
-  (movq (@ target::xcf.foreign-sp (% arg_z)) (% imm0))
-  (movq (% temp0) (:rcontext target::tcr.xframe))
-  (movq (% imm0) (:rcontext target::tcr.foreign-sp))
-  ;; All done processing the xcf.  NVRs may have been
-  ;; saved between the last catch/last xcf and the
-  ;; target frame.  The save-n-offset parameter/constants
-  ;; are either 0 or negative offsets from the target frame
-  ;; of the stack location where the corresponding GPR
-  ;; was saved.
-  @no-xcf
-  (movq (@ 'target-tsp (% fn)) (% imm0))
-  (cmpb ($ x8664::fulltag-nil) (%b imm0))
-  (movq (@ 'target-foreign-sp (% fn)) (% temp0))
-  (je @no-tsp)
-  (movq (% imm0) (:rcontext target::tcr.save-tsp))
-  (movq (% imm0) (:rcontext target::tcr.next-tsp))
-  @no-tsp
-  (cmpb ($ x8664::fulltag-nil) (%b temp0))
-  (je @no-sp)
-  (movq (% temp0) (:rcontext target::tcr.foreign-sp))
-  @no-sp
-  (movq (@ 'target-frame (% fn)) (% rbp))
-  (movq (@ 'save0-offset (% fn)) (% arg_x))
-  (movq (@ 'save1-offset (% fn)) (% arg_y))
-  (movq (@ 'save2-offset (% fn)) (% arg_z))
-  (movq (@ 'save3-offset (% fn)) (% temp0))
-  (testq (% arg_x) (% arg_x))
-  (cmovneq (@ (% rbp) (% arg_x)) (% save0))
-  (testq (% arg_y) (% arg_y))
-  (cmovneq (@ (% rbp) (% arg_x)) (% save1))
-  (testq (% arg_z) (% arg_z))
-  (cmovneq (@ (% rbp) (% arg_x)) (% save2))
-  (testq (% temp0) (% temp0))
-  (cmovneq (@ (% rbp) (% arg_x)) (% save3))
-  (leave)
-  (pop (% temp0))                       ; return address, not used by subprim
-  (set-nargs 0)
-  (movq (@ 'args (% fn)) (% arg_z))
-  (lea (@ (:^ @back-from-spread) (% fn)) (% ra0))
-  (:talign 4)
-  (jmp-subprim .SPspreadargz)
-  @back-from-spread
-  (recover-fn-from-rip)                 ; .SPspreadargz preserves %fn, but ...
-  (push (% temp0))                      ; return address
-  (jmp (@ 'function (% fn))))
-  
-
-
-
 
   
+
+
+
+
+;;; If x is a static cons, return its index value as a fixnum.
+;;; Otherwise return nil.
 (defx86lapfunction %staticp ((x arg_z))
   (check-nargs 1)
   (ref-global static-cons-area temp0)
@@ -1079,6 +969,9 @@
   (cmovaq (% imm1) (% arg_z))
   (single-value-return))
 
+;;; Return the static cons corresponding to the index n,
+;;; or nil if there's no such static cons (i.e., n is out
+;;; of range, or the static cons has been gc'd).
 (defx86lapfunction %static-inverse-cons ((n arg_z))
   (check-nargs 1)
   (testl ($ target::tagmask) (% arg_z.l))
