@@ -1,19 +1,18 @@
 ;;;-*-Mode: LISP; Package: CCL -*-
 ;;;
-;;;   Copyright (C) 2009 Clozure Associates
-;;;   Copyright (C) 1994-2001 Digitool, Inc
-;;;   This file is part of Clozure CL.  
+;;; Copyright 1994-2009 Clozure Associates
 ;;;
-;;;   Clozure CL is licensed under the terms of the Lisp Lesser GNU Public
-;;;   License , known as the LLGPL and distributed with Clozure CL as the
-;;;   file "LICENSE".  The LLGPL consists of a preamble and the LGPL,
-;;;   which is distributed with Clozure CL as the file "LGPL".  Where these
-;;;   conflict, the preamble takes precedence.  
+;;; Licensed under the Apache License, Version 2.0 (the "License");
+;;; you may not use this file except in compliance with the License.
+;;; You may obtain a copy of the License at
 ;;;
-;;;   Clozure CL is referenced in the preamble as the "LIBRARY."
+;;;     http://www.apache.org/licenses/LICENSE-2.0
 ;;;
-;;;   The LLGPL is also available online at
-;;;   http://opensource.franz.com/preamble.html
+;;; Unless required by applicable law or agreed to in writing, software
+;;; distributed under the License is distributed on an "AS IS" BASIS,
+;;; WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+;;; See the License for the specific language governing permissions and
+;;; limitations under the License.
 
 ; L1-utils.lisp
 
@@ -541,6 +540,10 @@ vector
 (defun bad-proclaim-spec (spec)
   (signal-program-error "Invalid declaration specifier ~s" spec))
 
+
+(defun using-linear-scan () *backend-use-linear-scan*)
+
+
 (defun proclaim-type (type &rest vars)
   (declare (dynamic-extent vars))
   ;; Called too early to use (every #'symbolp vars)
@@ -767,7 +770,9 @@ vector
         (integer (setq counter string-or-integer)) ; & emit-style-warning
         (string (setq prefix (ensure-simple-string string-or-integer)))))
     (unless counter
-      (with-lock-grabbed (*gensym-lock*)
+      (unless (and (numberp *gensym-counter*)(not (minusp *gensym-counter*)))
+        (error 'type-error :datum *gensym-counter* :expected-type 'unsigned-byte))
+      (with-lock-grabbed (*gensym-lock*) ; 
         (setq *gensym-counter* (1+ (setq counter *gensym-counter*)))))
     (make-symbol (%str-cat prefix (%integer-to-string counter)))))
 
