@@ -1,19 +1,18 @@
 ;;;-*-Mode: LISP; Package: CCL -*-
 ;;;
-;;;   Copyright (C) 2009 Clozure Associates
-;;;   Copyright (C) 1994-2001 Digitool, Inc
-;;;   This file is part of Clozure CL.  
+;;; Copyright 1994-2009 Clozure Associates
 ;;;
-;;;   Clozure CL is licensed under the terms of the Lisp Lesser GNU Public
-;;;   License , known as the LLGPL and distributed with Clozure CL as the
-;;;   file "LICENSE".  The LLGPL consists of a preamble and the LGPL,
-;;;   which is distributed with Clozure CL as the file "LGPL".  Where these
-;;;   conflict, the preamble takes precedence.  
+;;; Licensed under the Apache License, Version 2.0 (the "License");
+;;; you may not use this file except in compliance with the License.
+;;; You may obtain a copy of the License at
 ;;;
-;;;   Clozure CL is referenced in the preamble as the "LIBRARY."
+;;;     http://www.apache.org/licenses/LICENSE-2.0
 ;;;
-;;;   The LLGPL is also available online at
-;;;   http://opensource.franz.com/preamble.html
+;;; Unless required by applicable law or agreed to in writing, software
+;;; distributed under the License is distributed on an "AS IS" BASIS,
+;;; WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+;;; See the License for the specific language governing permissions and
+;;; limitations under the License.
 
 (in-package "CCL")
 
@@ -258,6 +257,8 @@
     defsystem
     jp-encode
     cn-encode
+    prefixed-stream
+    timestamped-stream
     ))
 
 
@@ -579,6 +580,12 @@ not runtime errors reported by a successfully created process."
 (defvar *build-time-optional-features* nil)
 (defvar *ccl-save-source-locations* :no-text)
 
+(defun report-build-environment (stream)
+  (format stream "~&Rebuilding ~a using ~a"
+          (lisp-implementation-type)
+          (lisp-implementation-version))
+  (format stream "~&Path to source code: ~s" (truename "ccl:")))
+
 (defun rebuild-ccl (&key update full clean kernel force (reload t) exit
                          reload-arguments verbose optional-features
                          (save-source-locations *ccl-save-source-locations*)
@@ -600,9 +607,7 @@ the lisp and run REBUILD-CCL again.")
 	  (return-from rebuild-ccl nil))))
     (when (or clean force)
       ;; for better bug reports...
-      (format t "~&Rebuilding ~a using ~a"
-              (lisp-implementation-type)
-              (lisp-implementation-version))
+      (report-build-environment t)
       (unless allow-constant-redefinition-p
         (when (or force clean update)
           (setq allow-constant-redefinition t))))
@@ -679,7 +684,7 @@ the lisp and run REBUILD-CCL again.")
                                 (eql exit-code 0))
                          (let* ((write-date (or (ignore-errors (file-write-date (standard-image-name))) 0)))
                            (unless (and write-date (> write-date old-write-date))
-                             (error "The hesp imsge ~a does not appear to have been written correctly.  This may indicate a problem with the bootstapping image." (standard-image-name)))
+                             (error "The heap image ~a does not appear to have been written correctly.  This may indicate a problem with the bootstapping image." (standard-image-name)))
                            (format t "~&;Wrote heap image: ~s"
                                    (truename (format nil "ccl:~a"
                                                      (standard-image-name))))
@@ -809,31 +814,36 @@ the lisp and run REBUILD-CCL again.")
 	      ,@body)
 	 (cwd ,wd)))))
 
+(defparameter *ccl-tests-directory* (make-pathname :directory
+                                                   (append (pathname-directory
+                                                            (translate-logical-pathname #P"ccl:"))
+                                                           '(:up "ccl-tests"))))
+
+(defparameter *ansi-tests-directory* (merge-pathnames  "ansi-tests/" *ccl-tests-directory*))
+
 (defun ensure-tests-loaded (&key force update ansi ccl (load t))
   (unless (and (find-package "REGRESSION-TEST") (not force))
-    (if (probe-file "ccl:tests;ansi-tests;")
-      (when update
-	(cwd "ccl:tests;")
-	(run-program *svn-program* '("update") :output t))
-      (let* ((repo (svn-repository))
-	     (url (format nil "~a/trunk/tests" repo))
-	     (s (make-string-output-stream)))
-	(if (null repo)
-	  (error "Can't determine svn repository.  ccl directory is ~s"
-		 (ccl-directory))
+    (if (probe-file *ansi-tests-directory*)
+        (when update
+          (cwd *ccl-tests-directory*)
+          (run-program "git" '("pull") :output t))
+        (let* ((giturl "git@github.com:Clozure/ccl-tests.git")
+               (s (make-string-output-stream)))
 	  (progn
 	    (format t "~&Using ~a to check out test suite from ~a ~
-                       into ccl:tests;~%" *svn-program* url)
-	    (cwd "ccl:")
+        into ~A~%" "git" giturl *ccl-tests-directory*)
+	    (cwd (make-pathname :directory (append (pathname-directory
+                                                    (translate-logical-pathname *ccl-tests-directory*))
+                                                   '(:up))))
 	    (multiple-value-bind (status exit-code)
-                (external-process-status
-                 (run-program *svn-program* (list "checkout" url "tests")
-                              :output s :error s))
+                                 (external-process-status
+                                  (run-program "git" (list "clone" giturl)
+                                               :output s :error s))
 	      (unless (and (eq status :exited)
 			   (eql exit-code 0))
 		(error "Failed to check out test suite: ~%~a"
-		       (get-output-stream-string s))))))))
-    (cwd "ccl:tests;ansi-tests;")
+		       (get-output-stream-string s)))))))
+    (cwd *ansi-tests-directory*)
     (run-program "make" '("-k" "clean") :output t)
     (map nil 'delete-file (directory "*.*fsl"))
     (when load
@@ -841,33 +851,34 @@ the lisp and run REBUILD-CCL again.")
       ;; it without making the test suite non-portable across platforms...
       (handler-bind ((warning (lambda (c)
                                 (if (typep c 'shadowed-typecase-clause)
-                                  (muffle-warning c)
-                                  (when (let ((w (or (and (typep c 'compiler-warning)
-                                                          (eq (compiler-warning-warning-type c) :program-error)
-                                                          (car (compiler-warning-args c)))
-                                                     c)))
-                                          (or (typep (car (compiler-warning-args c))
-                                                     'shadowed-typecase-clause)
-                                              (and (typep w 'simple-warning)
-                                                   (or 
-                                                    (string-equal
-                                                     (simple-condition-format-control w)
-                                                     "Clause ~S ignored in ~S form - shadowed by ~S .")
-                                                    ;; Might as well ignore these as well, they're intentional.
-                                                    (string-equal
-                                                     (simple-condition-format-control w)
-                                                     "Duplicate keyform ~s in ~s statement.")))))
-                                    (muffle-warning c))))))
+                                    (muffle-warning c)
+                                    (when (let ((w (or (and (typep c 'compiler-warning)
+                                                            (eq (compiler-warning-warning-type c) :program-error)
+                                                            (car (compiler-warning-args c)))
+                                                       c)))
+                                            (or (and (typep c 'compiler-warning)
+                                                     (typep (car (compiler-warning-args c))
+                                                       'shadowed-typecase-clause))
+                                                (and (typep w 'simple-warning)
+                                                     (or 
+                                                      (string-equal
+                                                       (simple-condition-format-control w)
+                                                       "Clause ~S ignored in ~S form - shadowed by ~S .")
+                                                      ;; Might as well ignore these as well, they're intentional.
+                                                      (string-equal
+                                                       (simple-condition-format-control w)
+                                                       "Duplicate keyform ~s in ~s statement.")))))
+                                      (muffle-warning c))))))
         ;; This loads the infrastructure
-        (load "ccl:tests;ansi-tests;gclload1.lsp")
+        (load (merge-pathnames *ansi-tests-directory* "gclload1.lsp"))
         ;; This loads the actual tests
         (let ((redef-var (find-symbol "*WARN-IF-REDEFINE-TEST*" :REGRESSION-TEST)))
           (progv (list redef-var) (list (if force nil (symbol-value redef-var)))
             (when ansi
-              (load "ccl:tests;ansi-tests;gclload2.lsp"))
+              (load (merge-pathnames *ansi-tests-directory* "gclload2.lsp")))
             ;; And our own tests
             (when ccl
-              (load "ccl:tests;ansi-tests;ccl.lsp"))))))))
+              (load (merge-pathnames *ansi-tests-directory* "ccl.lsp")))))))))
 
 
 (defun test-ccl (&key force (update t) verbose (catch-errors t) (ansi t) (ccl t)
@@ -906,7 +917,7 @@ the lisp and run REBUILD-CCL again.")
         (with-global-optimization-settings ()
           (proclaim `(optimize ,@optimization-settings))
           (ensure-tests-loaded :force force :update update :ansi ansi :ccl ccl)
-          (cwd "ccl:tests;ansi-tests;")
+          (cwd *ansi-tests-directory*)
           (let ((do-tests (find-symbol "DO-TESTS" "REGRESSION-TEST"))
                 (failed (find-symbol "*FAILED-TESTS*" "REGRESSION-TEST"))
                 (*print-catch-errors* nil))
@@ -916,7 +927,8 @@ the lisp and run REBUILD-CCL again.")
                                :optimization-settings (or optimization-settings '((speed 1) (space 1) (safety 1) (debug 1) (compilation-speed 1)))))
               ;; Clean up a little
               (map nil #'delete-file
-                   (directory (merge-pathnames *.fasl-pathname* "ccl:tests;ansi-tests;temp*"))))
+                   (directory (merge-pathnames *.fasl-pathname* (merge-pathnames *ansi-tests-directory*
+                                                                                 "temp*")))))
             (let ((failed-tests (symbol-value failed)))
               (when exit
                 (quit (if failed-tests 1 0)))
