@@ -1,4 +1,19 @@
 ;;;   -*- Mode: LISP; Syntax: Common-lisp; Package: (ANSI-LOOP "COMMON-LISP"); Base: 10; Lowercase:T -*-
+;;;
+;;; Copyright 1986-1992 Clozure Associates
+;;;
+;;; Licensed under the Apache License, Version 2.0 (the "License");
+;;; you may not use this file except in compliance with the License.
+;;; You may obtain a copy of the License at
+;;;
+;;;     http://www.apache.org/licenses/LICENSE-2.0
+;;;
+;;; Unless required by applicable law or agreed to in writing, software
+;;; distributed under the License is distributed on an "AS IS" BASIS,
+;;; WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+;;; See the License for the specific language governing permissions and
+;;; limitations under the License.
+;;;   -*- Mode: LISP; Syntax: Common-lisp; Package: (ANSI-LOOP "COMMON-LISP"); Base: 10; Lowercase:T -*-
 ;;;>
 ;;;> Portions of LOOP are Copyright (c) 1986 by the Massachusetts Institute of Technology.
 ;;;> All Rights Reserved.
@@ -641,14 +656,17 @@ a LET-like macro, and a SETQ-like macro, which perform LOOP-style destructuring.
 ;;;; Code Analysis Stuff
 
 
-(defun loop-constant-fold-if-possible (form &optional expected-type)
+(defun loop-constant-fold-if-possible (form &optional expected-type error-p)
   (let ((new-form form) (constantp nil) (constant-value nil))
     (when (setq constantp (constantp new-form))
       (setq constant-value (eval new-form)))
     (when (and constantp expected-type)
       (unless (typep constant-value expected-type)
-	(loop-warn "The form ~S evaluated to ~S, which was not of the anticipated type ~S."
-          form constant-value expected-type)
+        (if error-p
+            (loop-error "The form ~S evaluated to ~S, which was not of the anticipated type ~S."
+                       form constant-value expected-type)
+            (loop-warn "The form ~S evaluated to ~S, which was not of the anticipated type ~S."
+                       form constant-value expected-type))
 	(setq constantp nil constant-value nil)))
     (values new-form constantp constant-value)))
 
@@ -895,15 +913,19 @@ collected result will be returned as the value of the LOOP."
     ((atom tree) tree)
     (t (cons (subst-gensyms-for-nil (car tree))
 	     (subst-gensyms-for-nil (cdr tree))))))
- 
+
+(defmacro loop-destructuring-bind (lambda-list args-list &body body)
+  (let ((*ignores* ()))
+    (declare (special *ignores*))
+    (let ((dl (subst-gensyms-for-nil lambda-list)))
+      `(destructuring-bind (&optional ,@dl) ,args-list
+	 (declare (ignore ,@*ignores*))
+	 ,@body))))
+
 (defun loop-build-destructuring-bindings (crocks forms)
   (if crocks
-      (let ((*ignores* ()))
-	(declare (special *ignores*))
-	`((destructuring-bind ,(subst-gensyms-for-nil (car crocks))
-	      ,(cadr crocks)
-	    (declare (ignore ,@*ignores*))
-	    ,@(loop-build-destructuring-bindings (cddr crocks) forms))))
+      `((loop-destructuring-bind ,(car crocks) ,(cadr crocks)
+	,@(loop-build-destructuring-bindings (cddr crocks) forms)))
       forms))
 
 (defun loop-translate (*loop-source-code* *loop-macro-environment* *loop-universe*)
@@ -1819,7 +1841,7 @@ collected result will be returned as the value of the LOOP."
 	  (cond ((eq prep :downfrom) (setq dir ':down))
 		((eq prep :upfrom) (setq dir ':up)))
 	  (multiple-value-setq (form start-constantp start-value)
-	    (loop-constant-fold-if-possible form indexv-type))
+	    (loop-constant-fold-if-possible form))
 	  (setq indexv (loop-make-iteration-variable indexv form indexv-type)))
 	 ((:upto :to :downto :above :below)
 	  (cond ((loop-tequal prep :upto) (setq inclusive-iteration (setq dir ':up)))
@@ -1829,16 +1851,17 @@ collected result will be returned as the value of the LOOP."
 		((loop-tequal prep :below) (setq dir ':up)))
 	  (setq limit-given t)
 	  (multiple-value-setq (form limit-constantp limit-value)
-	    (loop-constant-fold-if-possible form indexv-type))
+	    (loop-constant-fold-if-possible form nil))
 	  (setq endform (if limit-constantp
 			    `',limit-value
 			    (loop-make-variable
-			      (loop-gentemp 'loop-limit-) form indexv-type))))
+			      (loop-gentemp 'loop-limit-) form nil))))
 	 (:by
-	   (multiple-value-setq (form stepby-constantp stepby)
-	     (loop-constant-fold-if-possible form indexv-type))
-	   (unless stepby-constantp
-	     (loop-make-variable (setq stepby (loop-gentemp 'loop-step-by-)) form indexv-type)))
+          (multiple-value-setq (form stepby-constantp stepby)
+            ;;; the by variable must be a positive number (so not zero)
+            (loop-constant-fold-if-possible form `(real (0)) t))
+          (unless stepby-constantp
+            (loop-make-variable (setq stepby (loop-gentemp 'loop-step-by-)) form `(real (0)))))
 	 (t (loop-error
 	      "~S invalid preposition in sequencing or sequence path.~@
 	       Invalid prepositions specified in iteration path descriptor or something?"
@@ -1858,7 +1881,7 @@ collected result will be returned as the value of the LOOP."
 	    (when (or limit-given default-top)
 	      (unless limit-given
 		(loop-make-variable (setq endform (loop-gentemp 'loop-seq-limit-))
-				    nil indexv-type)
+				    nil nil)
 		(push `(setq ,endform ,default-top) *loop-prologue*))
 	      (setq testfn (if inclusive-iteration '> '>=)))
 	    (setq step (if (eql stepby 1) `(1+ ,indexv) `(+ ,indexv ,stepby))))
@@ -1893,7 +1916,7 @@ collected result will be returned as the value of the LOOP."
       '((:from :upfrom :downfrom) (:to :upto :downto :above :below) (:by))
       nil (list (list kwd val)))))
 
-
+;; Seems to be unused
 (defun loop-sequence-elements-path (variable data-type prep-phrases
 				    &key fetch-function size-function sequence-type element-type)
   (multiple-value-bind (indexv indexv-user-specified-p) (named-variable 'index)
