@@ -1,18 +1,18 @@
 ;;;-*- Mode: Lisp; Package: CCL -*-
 ;;;
-;;;   Copyright (C) 2001-2014 Clozure Associates
-;;;   This file is part of Clozure CL.  
+;;; Copyright 2001-2014 Clozure Associates
 ;;;
-;;;   Clozure CL is licensed under the terms of the Lisp Lesser GNU Public
-;;;   License , known as the LLGPL and distributed with Clozure CL as the
-;;;   file "LICENSE".  The LLGPL consists of a preamble and the LGPL,
-;;;   which is distributed with Clozure CL as the file "LGPL".  Where these
-;;;   conflict, the preamble takes precedence.  
+;;; Licensed under the Apache License, Version 2.0 (the "License");
+;;; you may not use this file except in compliance with the License.
+;;; You may obtain a copy of the License at
 ;;;
-;;;   Clozure CL is referenced in the preamble as the "LIBRARY."
+;;;     http://www.apache.org/licenses/LICENSE-2.0
 ;;;
-;;;   The LLGPL is also available online at
-;;;   http://opensource.franz.com/preamble.html
+;;; Unless required by applicable law or agreed to in writing, software
+;;; distributed under the License is distributed on an "AS IS" BASIS,
+;;; WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+;;; See the License for the specific language governing permissions and
+;;; limitations under the License.
 
 (in-package "CCL")
 
@@ -43,6 +43,7 @@
             LOCAL-SOCKET-ADDRESS
 	    SOCKET-ADDRESS-FAMILY
 	    SOCKET-CONNECT
+            SOCKET-CONNECTED
 	    SOCKET-FORMAT
 	    SOCKET-TYPE
 	    SOCKET-ERROR
@@ -341,7 +342,8 @@ make-socket-args. The socket gets closed on exit."
   (:documentation "Return :internet, :internet6 or :file, as appropriate."))
 
 (defmethod socket-address-family ((socket ip-socket))
-  :internet)
+  ;; This is more expensive than one would like.
+  (socket-address-family (local-socket-address socket)))
 
 (defmethod socket-address-family ((socket file-socket))
   :file)
@@ -355,9 +357,10 @@ for udp-socket."))
 
 (defmethod socket-type ((socket stream-file-socket)) :stream)
 
+;;; Is this used by anybody?
 (defgeneric socket-connect (stream)
  (:documentation
-   "Return :active for tcp-stream, :passive for listener-socket, and NIL
+   "Return :active for tcp-stream, :passive for listener-socket, and socket-connected
 for udp-socket"))
 
 (defmethod socket-connect ((stream tcp-stream)) :active)
@@ -409,7 +412,7 @@ make-socket."))
   (assert (and in-p out-p) () "Non-bidirectional file-socket stream?")
   'basic-file-socket-stream)
 
-(defmethod socket-format ((socket unconnected-socket))
+(defmethod socket-format ((socket device-mixin))
   (or (getf (socket-keys socket) :format) :text))
 
 (defgeneric close (socket &key abort)
@@ -417,26 +420,39 @@ make-socket."))
    "The close generic function can be applied to sockets. It releases the
 operating system resources associated with the socket."))
 
-(defmethod close ((socket unconnected-socket) &key abort)
+(defmethod close ((socket device-mixin) &key abort)
   (declare (ignore abort))
   (when (socket-device socket)
     (fd-close (socket-device socket))
     (setf (socket-device socket) nil)
     t))
 
+(defmethod close :after ((socket udp-socket) &key abort)
+  (declare (ignore abort))
+  (setf (socket-connected socket) nil))
+
 (defmethod socket-connect ((stream listener-socket)) :passive)
 
 (defmethod socket-connect ((stream file-listener-socket)) :passive)
 
-(defun %socket-connect (fd socket-address &optional timeout-in-milliseconds)
-  (let ((err (c_connect fd (sockaddr socket-address) (sockaddr-length socket-address) timeout-in-milliseconds)))
+(defun %socket-connect (fd socket-address &optional timeout-msecs)
+  (let ((err (c_connect fd (sockaddr socket-address)
+                        (sockaddr-length socket-address) timeout-msecs)))
     (declare (fixnum err))
     (unless (eql err 0)
       #||(fd-close fd)||#
       (socket-error nil "connect" err nil :connect-address socket-address))))
 
+(defun %socket-bind (fd socket-address)
+  (let ((err (c_bind fd
+                     (sockaddr socket-address)
+                     (sockaddr-length socket-address))))
+    (declare (fixnum err))
+    (unless (eql err 0)
+      (socket-error nil "bind" err nil))))
+
 (defmethod socket-type ((stream udp-socket)) :datagram)
-(defmethod socket-connect ((stream udp-socket)) nil)
+(defmethod socket-connect ((stream udp-socket)) (socket-connected stream))
 (defmethod socket-format ((stream udp-socket)) :binary)
 
 (defgeneric socket-os-fd (socket)
@@ -504,75 +520,33 @@ the socket is not connected."))
   #-windows-target
   (not (logtest #$O_NONBLOCK (fd-get-flags fd))))
 
-(defun set-socket-options (socket
-                           &key 
-                             keepalive
-                             reuse-address
-                             nodelay
-                             broadcast
-                             linger
-                             (address-family :internet)
-                             local-port
-                             local-host
-                             local-address
-                             local-filename
-                             type
-                             connect
-                             out-of-band-inline
-			   &allow-other-keys)
+(defun %set-socket-options (fd &key keepalive reuse-address broadcast
+                                 out-of-band-inline linger
+                                 address-family type
+                                 nodelay
+                            &allow-other-keys)
   ;; see man socket(7) tcp(7) ip(7)
-  (let ((fd (socket-device socket)))
-    (when keepalive
-      (int-setsockopt fd #$SOL_SOCKET #$SO_KEEPALIVE 1))
-    (when reuse-address
-      (int-setsockopt fd #$SOL_SOCKET #$SO_REUSEADDR 1))
-    (when broadcast
-      (int-setsockopt fd #$SOL_SOCKET #$SO_BROADCAST 1))
-    (when out-of-band-inline
-      (int-setsockopt fd #$SOL_SOCKET #$SO_OOBINLINE 1))
-    (when (member address-family '(:internet :internet6))
-      (when (eq type :stream)
-	(rlet ((plinger :linger))
-	  (setf (pref plinger :linger.l_onoff) (if linger 1 0)
-		(pref plinger :linger.l_linger) (or linger 0))
-	  (socket-call socket "setsockopt"
-		       (c_setsockopt fd #$SOL_SOCKET #$SO_LINGER
-				     plinger (record-length :linger)))))
-      (when nodelay
-	(int-setsockopt fd
-			#+linux-target #$SOL_TCP
-			#-linux-target #$IPPROTO_TCP
-			#$TCP_NODELAY 1))
-      (when (or local-port local-host local-address)
-        (socket-bind-local socket (or local-address
-                                      (resolve-address :host local-host
-                                                       :port local-port
-                                                       :connect connect
-                                                       :address-family address-family
-                                                       :socket-type type)))))
-    (when (and (eq address-family :file)
-	       (eq connect :passive))
-      (unless local-filename
-        (error "need :local-filename argument to create passive file socket"))
-      #+windows-target (error "can't create file socket on Windows")
-      #-windows-target (socket-bind-local socket (make-instance 'unix-socket-address :path local-filename)))))
-
-;; I hope the inline declaration makes the &rest/apply's go away...
-(declaim (inline make-ip-socket))
-(defun make-ip-socket (&rest keys &key (type :stream) &allow-other-keys)
-  (declare (dynamic-extent keys))
-  (ecase type
-    (:stream (apply #'make-tcp-socket keys))
-    (:datagram (apply #'make-udp-socket keys))))
-
-#-windows-target
-(declaim (inline make-file-socket))
-#-windows-target
-(defun make-file-socket (&rest keys &key type &allow-other-keys)
-  (declare (dynamic-extent keys))
-  (ecase type
-    ((nil :stream) (apply #'make-stream-file-socket keys))
-    (:datagram (apply #'make-datagram-file-socket keys))))
+  (when keepalive
+    (int-setsockopt fd #$SOL_SOCKET #$SO_KEEPALIVE 1))
+  (when reuse-address
+    (int-setsockopt fd #$SOL_SOCKET #$SO_REUSEADDR 1))
+  (when broadcast
+    (int-setsockopt fd #$SOL_SOCKET #$SO_BROADCAST 1))
+  (when out-of-band-inline
+    (int-setsockopt fd #$SOL_SOCKET #$SO_OOBINLINE 1))
+  (when linger
+    (check-type linger (signed-byte 32))
+    (rlet ((plinger :linger))
+      (setf (pref plinger :linger.l_onoff) 1
+            (pref plinger :linger.l_linger) linger)
+      (let ((err (c_setsockopt fd #$SOL_SOCKET #$SO_LINGER
+			       plinger (record-length :linger))))
+        (unless (eql err 0)
+          (socket-error nil "setsockopt/linger" fd nil)))))
+  (when (and (memq address-family '(:internet :internet6))
+             (eq type :stream))
+    (when nodelay
+      (int-setsockopt fd #$IPPROTO_TCP #$TCP_NODELAY 1))))
 
 (defun make-socket (&rest keys
 		    &key
@@ -590,7 +564,7 @@ the socket is not connected."))
                       fd)
   "Create and return a new socket."
   (declare (dynamic-extent keys))
-  (declare (ignore type connect remote-host remote-port remote-address
+  (declare (ignore connect remote-host remote-port remote-address
 		   eol format keepalive reuse-address nodelay
 		   broadcast linger local-port local-host
 		   local-address backlog class out-of-band-inline
@@ -599,41 +573,71 @@ the socket is not connected."))
 		   input-timeout output-timeout deadline fd))
   (ecase address-family
     #-windows-target
-    ((:file) (apply #'make-file-socket keys))
-    ((:internet :internet6) (apply #'make-ip-socket keys))))
+    ((:file)
+     (ecase type
+       ((nil :stream) (apply #'make-stream-file-socket keys))
+       (:datagram (error "Datagram file sockets aren't implemented."))))
+    ((:internet :internet6)
+     (ecase type
+       (:stream (apply #'make-tcp-socket keys))
+       (:datagram (apply #'make-udp-socket keys))))))
 
 (defun make-udp-socket (&rest keys
                         &key
                           (fd -1)
                           (connect :active)
                           (address-family :internet)
-                          remote-host remote-port
+                          (type :datagram)
+                          remote-host remote-port remote-address
+                          local-host local-port local-address
                         &allow-other-keys)
+  (assert (eq type :datagram))
   (unwind-protect
-    (let (socket)
-      (when (< fd 0)
-        (setq fd (socket-call nil "socket"
-                              (c_socket (ecase address-family
-                                          (:internet #$PF_INET)
-                                          (:internet6 #$PF_INET6))
-                                        #$SOCK_DGRAM #$IPPROTO_UDP))))
-      (setq socket (make-instance 'udp-socket
-				  :device fd
-				  :keys keys))
-      (apply #'set-socket-options socket keys)
-      (when (and (eql connect :active)
-                 remote-host remote-port)
-        (%socket-connect fd
-                         (apply #'resolve-address
-				:connect connect
-				:address-family address-family
-                                :host remote-host
-                                :port remote-port
-                                :allow-other-keys t
-                                keys)
-                         nil))
-      (setq fd -1)
-      socket)
+       (let (socket connectedp)
+         (when (< fd 0)
+           (setq fd (socket-call nil "socket"
+                                 (c_socket (ecase address-family
+                                             (:internet #$PF_INET)
+                                             (:internet6 #$PF_INET6))
+                                           #$SOCK_DGRAM #$IPPROTO_UDP))))
+         (apply #'%set-socket-options fd keys)
+         (when (or local-address local-port local-host)
+           (let ((local (or local-address
+                            (resolve-address :host local-host
+                                             :port local-port
+                                             ;; Specify :passive
+                                             ;; because we want a
+                                             ;; socket address ready
+                                             ;; for use with bind.
+                                             ;; See note about
+                                             ;; AI_PASSIVE in
+                                             ;; getaddrinfo(3)
+                                             :connect :passive
+                                             :address-family address-family
+                                             :socket-type type))))
+             (%socket-bind fd local)))
+         (when (and (eq connect :active)
+                    (or remote-address
+                        (and remote-host remote-port)))
+           (let ((remote (or remote-address
+                             (resolve-address :host remote-host
+                                              :port remote-port
+                                              ;; Specify :active
+                                              ;; because we want a
+                                              ;; socket address ready
+                                              ;; for use with connect.
+                                              :connect :active
+                                              :address-family address-family
+                                              :socket-type type))))
+             (%socket-connect fd remote nil)
+             (setq connectedp t)))
+         ;; Now that the fd is all set up, make the socket object.
+         (setq socket (make-instance 'udp-socket
+                                     :device fd
+                                     :keys keys))
+         (setf (socket-connected socket) connectedp)
+         (setq fd -1)
+         socket)
     (unless (< fd 0)
       (fd-close fd))))
 
@@ -642,44 +646,59 @@ the socket is not connected."))
                           (connect :active)
                           (fd -1)
                           (address-family :internet)
+                          (type :stream)
+                          local-host local-port local-address
                           remote-host remote-port remote-address
                           backlog connect-timeout deadline
                         &allow-other-keys)
+  (assert (eq type :stream))
   (unwind-protect
-       (let ((timeout-in-milliseconds
-               (cond
-                 (deadline
-                  (max (round (- deadline (get-internal-real-time))
-                              (/ internal-time-units-per-second 1000))
-                       0))
-                 (connect-timeout
-		  (check-io-timeout connect-timeout)
-                  (round (* connect-timeout 1000)))))
-             (socket-address (or remote-address
-                                 (apply #'resolve-address
-					:connect connect
-					:address-family address-family
-                                        :host remote-host
-                                        :port remote-port
-                                        :allow-other-keys t
-                                        keys))))
-         (when (< fd 0)
+       (let (socket)
+	 (when (< fd 0)
            (setq fd (socket-call nil "socket"
                                  (c_socket (ecase address-family
                                              (:internet #$PF_INET)
                                              (:internet6 #$PF_INET6))
                                            #$SOCK_STREAM #$IPPROTO_TCP))))
-         (let ((socket (apply (ecase connect
-                                (:active #'make-tcp-stream-socket)
-                                (:passive #'make-tcp-listener-socket))
-                              fd
-                              keys)))
-           (apply #'set-socket-options socket keys)
-           (if (eql connect :passive)
-               (socket-call nil "listen" (c_listen fd (or backlog 5)))
-               (%socket-connect fd socket-address timeout-in-milliseconds))
-           (setq fd -1)
-           socket))
+         (apply #'%set-socket-options fd keys)
+         (when (or local-port local-host local-address)
+           (let ((local (or local-address
+                            (resolve-address :host local-host
+                                             :port local-port
+                                             ;; want socket addr for bind
+                                             :connect :passive
+                                             :address-family address-family
+                                             :socket-type type))))
+             (%socket-bind fd local)))
+         (ecase connect
+           (:passive
+            (socket-call nil "listen" (c_listen fd (or backlog 5))))
+           (:active
+            (let ((timeout-msecs
+		    (cond
+		      (deadline
+		       (max (round (- deadline (get-internal-real-time))
+				   (/ internal-time-units-per-second 1000))
+			    0))
+		      (connect-timeout
+		       (check-io-timeout connect-timeout)
+		       (round (* connect-timeout 1000)))))
+		  (remote (or remote-address
+			      (resolve-address :host remote-host
+				               :port remote-port
+                                               ;; want socket addr for connect
+                                               :connect :active
+				               :address-family address-family
+                                               :socket-type type))))
+              (%socket-connect fd remote timeout-msecs))))
+         ;; Now that the fd is all set up, make the socket object.
+         (setq socket (apply (ecase connect
+                               (:active #'make-tcp-stream-socket)
+                               (:passive #'make-tcp-listener-socket))
+                             fd
+                             keys))
+         (setq fd -1)
+         socket)
     (unless (< fd 0)
       (fd-close fd))))
 
@@ -698,50 +717,47 @@ the socket is not connected."))
                                   (connect :active)
                                   backlog
                                   (fd -1)
+                                  local-filename
+                                  remote-filename
                                 &allow-other-keys)
   (unwind-protect
        (let (socket)
          (when (< fd 0)
-           (setq fd (socket-call nil "socket" (c_socket #$PF_LOCAL #$SOCK_STREAM 0))))
-         (setq socket
-                (apply (ecase connect
-                         (:active #'make-file-stream-socket)
-                         (:passive #'make-file-listener-socket))
-                       fd keys))
-         (apply #'set-socket-options socket keys)
-         (when (eql connect :passive)
-           (socket-call nil "listen" (c_listen fd (or backlog 5))))
+           (setq fd (socket-call nil "socket" (c_socket #$PF_LOCAL
+                                                        #$SOCK_STREAM 0))))
+         (apply #'%set-socket-options fd keys)
+         (ecase connect
+           (:passive
+            (unless local-filename
+              (error "need :local-filename argument to create server file ~
+                       stream socket"))
+            (let ((local (make-instance 'unix-socket-address
+                                        :path local-filename)))
+              (%socket-bind fd local)
+              (socket-call nil "listen" (c_listen fd (or backlog 5)))))
+           (:active
+            (unless remote-filename
+              (error "need :remote-filename argument to create client file ~
+                       stream socket"))
+            (let ((remote (make-instance 'unix-socket-address
+                                         :path remote-filename)))
+              (%socket-connect fd remote))))
+         (setq socket (apply (ecase connect
+                               (:active #'make-file-stream-socket)
+                               (:passive #'make-file-listener-socket))
+                             fd keys))
          (setq fd -1)
          socket)
     (unless (< fd 0)
       (fd-close fd))))
 
-(defun make-datagram-file-socket (&rest keys)
-  (declare (ignore keys))
-  (error "Datagram file sockets aren't implemented."))
-
-#-windows-target
-(defun file-socket-connect (fd remote-filename)
-  (%socket-connect fd (make-instance 'unix-socket-address :path remote-filename)))
-
-#+windows-target
-(defun file-socket-connect (fd remote-filename)
-  (declare (ignore fd))
-  (error "Can't create file socket to ~s on Windows" remote-filename))
-  
 (defun make-tcp-stream-socket (fd &rest keys &key &allow-other-keys)
   (apply #'make-tcp-stream fd keys))
 
 #-windows-target
-(defun make-file-stream-socket (fd &rest keys
-                                   &key remote-filename
-                                   &allow-other-keys)
-  (unless remote-filename
-    (error "need :remote-filename argument when creating file stream socket"))
-  (file-socket-connect fd remote-filename)
+(defun make-file-stream-socket (fd &rest keys &key &allow-other-keys)
   (apply #'make-file-socket-stream fd keys))
 
-
 (defun make-tcp-stream (fd
                         &key (format :bivalent)
                              external-format
@@ -843,7 +859,8 @@ the socket is not connected."))
 	      (_accept fd *multiprocessing-socket-io*)))
 	  (*multiprocessing-socket-io*
 	    (_accept fd t))
-	  (t
+	  (t ; if nowait was specified, temporarily force the socket to not block
+             ;  (sockets are generally 'born blocking' in CCL)
 	    (let ((was-blocking (get-socket-fd-blocking fd)))
 	      (unwind-protect
 		  (progn
@@ -908,24 +925,29 @@ accept-connection on it again."))
 (defmethod send-to ((socket udp-socket) msg size
 		    &key remote-host remote-port remote-address offset)
   "Send a UDP packet over a socket."
-  (let ((socket-address (or remote-address
-                            (remote-socket-address socket)
-                            (resolve-address :host (or remote-host
-                                                       (getf (socket-keys socket) :remote-host))
-                                             :port (or remote-port
-                                                       (getf (socket-keys socket) :remote-port))
-                                             :connect :active
-                                             :address-family (socket-address-family socket)
-                                             :socket-type :datagram))))
+  (let ((socket-address (and (not (socket-connected socket))
+                             (or remote-address
+                                 (remote-socket-address socket)
+                                 (resolve-address :host (or remote-host
+                                                            (getf (socket-keys socket) :remote-host))
+                                                  :port (or remote-port
+                                                            (getf (socket-keys socket) :remote-port))
+                                                  :connect :active
+                                                  :address-family (socket-address-family socket)
+                                                  :socket-type :datagram)))))
     (multiple-value-setq (msg offset) (verify-socket-buffer msg offset size))
     (%stack-block ((bufptr size))
       (%copy-ivector-to-ptr msg offset bufptr 0 size)
       (socket-call socket "sendto"
-        (with-eagain (socket-device socket) :output
-          (c_sendto (socket-device socket)
-                    bufptr size 0
-                    (sockaddr socket-address)
-                    (sockaddr-length socket-address)))))))
+                   (with-eagain (socket-device socket) :output
+                     (c_sendto (socket-device socket)
+                               bufptr size 0
+                               (if socket-address
+                                   (sockaddr socket-address)
+                                   (%null-ptr))
+                               (if socket-address
+                                   (sockaddr-length socket-address)
+                                   0)))))))
 
 (defmethod receive-from ((socket udp-socket) size &key buffer extract offset want-socket-address-p)
   "Read a UDP packet from a socket. If no packets are available, wait for
@@ -942,7 +964,7 @@ If want-socket-address-p is true:
 
   The buffer with the data
   The number of bytes read
-  The socket-address describing the sender
+  The socket-address object describing the sender
 "
   (let ((fd (socket-device socket))
 	(vec-offset offset)
@@ -990,8 +1012,12 @@ you need to read responses after sending an end-of-file signal."))
   (let ((fd (socket-device socket)))
     (socket-call socket "shutdown"
       (c_shutdown fd (ecase direction
-		       (:input 0)
-		       (:output 1))))))
+		       (:input #-windows-target #$SHUT_RD
+			       #+windows-target #$SD_RECEIVE)
+		       (:output #-windows-target #$SHUT_WR
+				#+windows-target #$SD_SEND)
+                       (:both #-windows-target #$SHUT_RDWR
+			      #+windows-target #$SD_BOTH))))))
 
 (defun dotted-to-ipaddr (name &key (errorp t))
   "Convert a dotted-string representation of a host address to a 32-bit
@@ -1028,20 +1054,17 @@ unsigned IP address."
 	(socket-error socket "getsockopt" err nil)))))
 
 (defun timeval-setsockopt (socket level optname timeout)
-    (multiple-value-bind (seconds micros)
-        (microseconds timeout)
-      (rlet ((valptr :timeval :tv_sec seconds :tv_usec micros))
-        (socket-call socket "setsockopt"
-          (c_setsockopt socket level optname valptr (record-length :timeval))))))
-                   
-(defun int-setsockopt (socket level optname optval)
+  (multiple-value-bind (seconds micros)
+      (microseconds timeout)
+    (rlet ((valptr :timeval :tv_sec seconds :tv_usec micros))
+      (socket-call socket "setsockopt"
+        (c_setsockopt socket level optname valptr (record-length :timeval))))))
+
+(defun int-setsockopt (fd level optname optval)
   (rlet ((valptr :signed))
     (setf (pref valptr :signed) optval)
-    (socket-call socket "setsockopt"
-      (c_setsockopt socket level optname valptr (record-length :signed)))))
-
-
-
+    (socket-call nil (format nil "setsockopt ~d/~d" optname optval)
+      (c_setsockopt fd level optname valptr (record-length :signed)))))
             
 (defun c_gethostbyaddr (addr-in-net-byte-order)
   (rletZ ((sin #>sockaddr_in))
@@ -1463,25 +1486,25 @@ unsigned IP address."
       (princ-to-string port)))
 
 (defun resolve-address (&key
-                          host
-                          port
-                          (socket-type :stream)
-                          (connect :active)
-                          address-family
-                          numeric-host-p
-                          #-windows-target numeric-service-p
-                          (singlep t)
-                          (errorp t))
+                        host
+                        port
+                        (socket-type :stream)
+                        (connect :active)
+                        address-family
+                        numeric-host-p
+                        #-windows-target numeric-service-p
+                        (singlep t)
+                        (errorp t))
   "Resolve a host and/or port string to one or more socket-address
-instances.  Either host or port may be unspecified.  Calls
-getaddrinfo() underneath.
-
-singlep may be passed as NIL to make the function return a list of
-host addresses matching the specified query terms.  The default is to
-return the first matching address.
-
-errorp may be passed as NIL to return NIL if no match was found."
-
+  instances.  Either host or port may be unspecified.  Calls
+  getaddrinfo() underneath.
+  
+  singlep may be passed as NIL to make the function return a list of
+  host addresses matching the specified query terms.  The default is to
+  return the first matching address.
+  
+  errorp may be passed as NIL to return NIL if no match was found."
+  
   ;; We have historically supported the use of an (unsigned-byte 32)
   ;; value to represent an IPv4 address. If existing code does that to
   ;; avoid overhead (name resolution, consing, what-have-you), then
@@ -1497,18 +1520,18 @@ errorp may be passed as NIL to return NIL if no match was found."
 			(symbol (_getservbyname (string-downcase
 						 (string port)) proto)))))
       (if (null inet-port)
-	(when errorp
-	  (error "can't resolve port ~s with getservbyname" port))
-	(let* ((socket-address (make-instance 'socket-address))
-	       (sin (sockaddr socket-address)))
-	  (setf (pref sin :sockaddr_in.sin_family) #$AF_INET)
-	  (setf (pref sin
-		      #+(or windows-target solaris-target) #>sockaddr_in.sin_addr.S_un.S_addr
-		      #-(or windows-target solaris-target) :sockaddr_in.sin_addr.s_addr) (htonl host))
-	  (setf (pref sin :sockaddr_in.sin_port) inet-port)
-	  (upgrade-socket-address-from-sockaddr #$AF_INET socket-address)
-	  (return-from resolve-address socket-address)))))
-    
+          (when errorp
+            (error "can't resolve port ~s with getservbyname" port))
+          (let* ((socket-address (make-instance 'socket-address))
+                 (sin (sockaddr socket-address)))
+            (setf (pref sin :sockaddr_in.sin_family) #$AF_INET)
+            (setf (pref sin
+                        #+(or windows-target solaris-target) #>sockaddr_in.sin_addr.S_un.S_addr
+                        #-(or windows-target solaris-target) :sockaddr_in.sin_addr.s_addr) (htonl host))
+            (setf (pref sin :sockaddr_in.sin_port) inet-port)
+            (upgrade-socket-address-from-sockaddr #$AF_INET socket-address)
+            (return-from resolve-address socket-address)))))
+  
   (with-cstrs ((host-buf (or host ""))
                (port-buf (string-downcase (or (ensure-string port) ""))))
     (rletZ ((hints #>addrinfo)
@@ -1535,24 +1558,26 @@ errorp may be passed as NIL to return NIL if no match was found."
                                  results)))
         (if (eql 0 err)
             (prog1
-                (or (loop for info = (pref results :address) then (pref info #>addrinfo.ai_next)
-                          until (%null-ptr-p info)
-                          for sockaddr = (pref info #>addrinfo.ai_addr)
-                          for socket-address = (make-instance 'socket-address)
-                          do (loop for i below (pref info #>addrinfo.ai_addrlen)
-                                   do (setf (paref (sockaddr socket-address) :uint8_t i)
-                                            (paref sockaddr :uint8_t i)))
-                             (upgrade-socket-address-from-sockaddr (pref (sockaddr socket-address) :sockaddr_storage.ss_family)
-                                                                   socket-address)
-                          if singlep
-                            do (return socket-address)
-                          else
-                            collect socket-address)
-                    (when errorp
-                      (error "cannot resolve local service host ~A port ~A connect ~S type ~S"
-                             host port connect socket-type)))
+              (or (loop for info = (pref results :address) then (pref info #>addrinfo.ai_next)
+                    until (%null-ptr-p info)
+                    for sockaddr = (pref info #>addrinfo.ai_addr)
+                    for socket-address = (make-instance 'socket-address)
+                    do (loop for i below (pref info #>addrinfo.ai_addrlen)
+                         do (setf (paref (sockaddr socket-address) :uint8_t i)
+                                  (paref sockaddr :uint8_t i)))
+                    (upgrade-socket-address-from-sockaddr (pref (sockaddr socket-address) :sockaddr_storage.ss_family)
+                                                          socket-address)
+                    if singlep
+                    do (return socket-address)
+                    else
+                    collect socket-address)
+                  (when errorp
+                    (error "cannot resolve local service host ~A port ~A connect ~S type ~S"
+                           host port connect socket-type)))
               (#_freeaddrinfo (pref results :address)))
-	      (values nil err))))))
+            (if errorp 
+                (socket-error nil "getaddrinfo" err t) 
+                (values nil err)))))))
 
 (defclass ip4-socket-address (ip-socket-address)
   ())
@@ -1658,11 +1683,6 @@ the resulting sockaddr."
                           #-darwin-target
                           (%get-cstring (pref sockaddr :sockaddr_un.sun_path))))))
 
-(defmethod socket-bind-local (socket socket-address)
-  (socket-call socket "bind" (c_bind (socket-device socket)
-                                     (sockaddr socket-address)
-                                     (sockaddr-length socket-address))))
-
 (defun get-socket-address-from-call (socket function call-name)
   (let ((socket-address (make-instance 'socket-address)))
     (rlet ((namelen :signed (sockaddr-length socket-address)))
@@ -1708,7 +1728,8 @@ the resulting sockaddr."
                                                               (case (find-symbol (string-upcase proto) :keyword)
                                                                 (:udp :datagram)
                                                                 (:tcp :stream))
-                                                              proto))))
+                                                              proto)
+                                             :errorp nil)))
         (when socket-address
           (port socket-address)))))
 
@@ -1717,7 +1738,7 @@ the resulting sockaddr."
 unsigned IP address."
   (if (typep host 'integer)
       host
-      (let ((socket-address (resolve-address :host host :address-family :internet)))
+      (let ((socket-address (resolve-address :host host :address-family :internet :errorp nil)))
         (when socket-address
           (host socket-address)))))
 
