@@ -1,19 +1,18 @@
 /*
-   Copyright (C) 2009 Clozure Associates
-   Copyright (C) 1994-2001 Digitool, Inc
-   This file is part of Clozure CL.  
-
-   Clozure CL is licensed under the terms of the Lisp Lesser GNU Public
-   License , known as the LLGPL and distributed with Clozure CL as the
-   file "LICENSE".  The LLGPL consists of a preamble and the LGPL,
-   which is distributed with Clozure CL as the file "LGPL".  Where these
-   conflict, the preamble takes precedence.  
-
-   Clozure CL is referenced in the preamble as the "LIBRARY."
-
-   The LLGPL is also available online at
-   http://opensource.franz.com/preamble.html
-*/
+ * Copyright 1994-2009 Clozure Associates
+ *
+ * Licensed under the Apache License, Version 2.0 (the "License");
+ * you may not use this file except in compliance with the License.
+ * You may obtain a copy of the License at
+ *
+ * http://www.apache.org/licenses/LICENSE-2.0
+ *
+ * Unless required by applicable law or agreed to in writing, software
+ * distributed under the License is distributed on an "AS IS" BASIS,
+ * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+ * See the License for the specific language governing permissions and
+ * limitations under the License.
+ */
 
 #include "lisp.h"
 #include "lisp-exceptions.h"
@@ -46,14 +45,14 @@ typedef enum {
   debug_kill
 } debug_command_return;
 
-#ifdef SVN_REVISION
+#ifdef VC_REVISION
 #define xstr(s) str(s)
 #define str(s) #s
-char *kernel_svn_revision = xstr(SVN_REVISION);
+char *kernel_vc_revision = xstr(VC_REVISION);
 #undef xstr
 #undef str
 #else
-char *kernel_svn_revision = "unknown";
+char *kernel_vc_revision = "unknown";
 #endif
 
 #ifdef ARM
@@ -209,6 +208,7 @@ foreign_name_and_offset(natural addr, int *delta)
 
 
 #if defined(LINUX) || defined(SOLARIS)
+#include <stdio_ext.h>
 #define fpurge __fpurge
 #endif
 
@@ -710,17 +710,17 @@ char *
 area_code_name(int code)
 {
   switch (code) {
-    case AREA_VOID: return "void";
-    case AREA_CSTACK: return "cstack";
-    case AREA_VSTACK: return "vstack";
-    case AREA_TSTACK: return "tstack";
-    case AREA_READONLY: return "readonly";
-    case AREA_WATCHED: return "watched";
-    case AREA_STATIC_CONS: return "static cons";
-    case AREA_MANAGED_STATIC: return "managed static";
-    case AREA_STATIC: return "static";
-    case AREA_DYNAMIC: return "dynamic";
-    default: return "unknown";
+    case AREA_VOID: return "--";
+    case AREA_CSTACK: return "cs";
+    case AREA_VSTACK: return "vs";
+    case AREA_TSTACK: return "ts";
+    case AREA_READONLY: return "ro";
+    case AREA_WATCHED: return "wa";
+    case AREA_STATIC_CONS: return "sc";
+    case AREA_MANAGED_STATIC: return "ms";
+    case AREA_STATIC: return "st";
+    case AREA_DYNAMIC: return "dy";
+    default: return "??";
   }
 }
 
@@ -731,11 +731,12 @@ debug_memory_areas(ExceptionInformation *xp, siginfo_t *info, int arg)
   char label[100];
 
   fprintf(dbgout, "Lisp memory areas:\n");
-  fprintf(dbgout, "%20s %20s %20s\n", "code", "low", "high");
+  fprintf(dbgout, "%21s %19s %19s %17s\n", "code", "low", "high", "size");
   for (a = header->succ; a != header; a = a->succ) {
-    snprintf(label, sizeof(label), "%s (%d)", area_code_name(a->code),
+    snprintf(label, sizeof(label), "%p %s (%d)", a, area_code_name(a->code),
 	     a->code >> fixnumshift);
-    fprintf(dbgout, "%20s %20p %20p\n", label, a->low, a->high);
+    fprintf(dbgout, "%21s %19p %19p %17#x\n", label, a->low, a->high,
+      a->high - a->low);
   }
   return debug_continue;
 }
@@ -779,10 +780,10 @@ debug_lisp_registers(ExceptionInformation *xp, siginfo_t *info, int arg)
     fprintf(dbgout,"------\n");
     show_lisp_register(xp, "fn", Ifn);
     fprintf(dbgout,"------\n");
-    show_lisp_register(xp, "save0", Isave0);
-    show_lisp_register(xp, "save1", Isave1);
-    show_lisp_register(xp, "save2", Isave2);
-    show_lisp_register(xp, "save3", Isave3);
+    show_lisp_register(xp, "temp6", Itemp6);
+    show_lisp_register(xp, "temp5", Itemp5);
+    show_lisp_register(xp, "temp4", Itemp4);
+    show_lisp_register(xp, "temp3", Itemp3);
     fprintf(dbgout,"------\n");
     show_lisp_register(xp, "temp0", Itemp0);
     show_lisp_register(xp, "temp1", Itemp1);
@@ -808,6 +809,7 @@ debug_lisp_registers(ExceptionInformation *xp, siginfo_t *info, int arg)
 #endif
 #ifdef ARM
     TCR *xpcontext = (TCR *)ptr_from_lispobj(xpGPR(xp, rcontext));
+    extern Boolean active_tcr_p(TCR *);
 
     fprintf(dbgout, "rcontext = 0x%lX ", xpcontext);
     if (!active_tcr_p(xpcontext)) {
@@ -946,7 +948,7 @@ debug_show_lisp_version(ExceptionInformation *xp, siginfo_t *info, int arg)
 {
   extern void *plsym(ExceptionInformation *,char*);
 
-  fprintf(dbgout, "Lisp kernel svn revision: %s\n", kernel_svn_revision);
+  fprintf(dbgout, "Lisp kernel vc revision: %s\n", kernel_vc_revision);
   if (xp)
     plsym(xp, "*OPENMCL-VERSION*");
   return debug_continue;
