@@ -1,19 +1,18 @@
 /*
-   Copyright (C) 2009 Clozure Associates
-   Copyright (C) 1994-2001 Digitool, Inc
-   This file is part of Clozure CL.  
-
-   Clozure CL is licensed under the terms of the Lisp Lesser GNU Public
-   License , known as the LLGPL and distributed with Clozure CL as the
-   file "LICENSE".  The LLGPL consists of a preamble and the LGPL,
-   which is distributed with Clozure CL as the file "LGPL".  Where these
-   conflict, the preamble takes precedence.  
-
-   Clozure CL is referenced in the preamble as the "LIBRARY."
-
-   The LLGPL is also available online at
-   http://opensource.franz.com/preamble.html
-*/
+ * Copyright 1994-2009 Clozure Associates
+ *
+ * Licensed under the Apache License, Version 2.0 (the "License");
+ * you may not use this file except in compliance with the License.
+ * You may obtain a copy of the License at
+ *
+ * http://www.apache.org/licenses/LICENSE-2.0
+ *
+ * Unless required by applicable law or agreed to in writing, software
+ * distributed under the License is distributed on an "AS IS" BASIS,
+ * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+ * See the License for the specific language governing permissions and
+ * limitations under the License.
+ */
 
 
 #include "threads.h"
@@ -599,7 +598,7 @@ suspend_resume_handler(int signo, siginfo_t *info, ExceptionInformation *context
     *(--tcr->save_vsp) = lisp_nil;
     register_thread_tcr(tcr);
   }
-  if (TCR_INTERRUPT_LEVEL(tcr) <= (-2<<fixnumshift)) {
+  if (TCR_INTERRUPT_LEVEL(tcr) <= -(2 << fixnumshift)) {
     SET_TCR_FLAG(tcr,TCR_FLAG_BIT_PENDING_SUSPEND);
   } else {
     TCR_AUX(tcr)->suspend_context = context;
@@ -663,6 +662,11 @@ os_get_current_thread_stack_bounds(void **base, natural *size)
 #endif
 #ifdef SOLARIS
   stack_t st;
+  /*
+   * If we #include <thread.h> then a conflicting definition
+   * for rwlock_destroy comes into scope.
+   */
+  extern int thr_stksegment(stack_t *ss);
   
   thr_stksegment(&st);
   *size = st.ss_size;
@@ -867,6 +871,13 @@ setup_tcr_extra_segment(TCR *tcr)
   amd64_set_gsbase(tcr);
 #endif
 #ifdef LINUX
+  /*
+   * According arch_prctl(2), there's no function prototype for
+   * arch_prctl().  Thus, we have to declare it ourselves.
+   * Note that addr is unsigned long for SET operations.
+   */
+  extern int arch_prctl(int code, unsigned long addr);
+
   arch_prctl(ARCH_SET_GS, (natural)tcr);
 #endif
 #ifdef DARWIN
@@ -1329,7 +1340,7 @@ new_tcr(natural vstack_size, natural tstack_size)
   for (i = 0; i < 2048; i++) {
     tcr->tlb_pointer[i] = (LispObj) no_thread_local_binding_marker;
   }
-  TCR_INTERRUPT_LEVEL(tcr) = (LispObj) (-1<<fixnum_shift);
+  TCR_INTERRUPT_LEVEL(tcr) = (LispObj) -(1 << fixnumshift);
 #ifndef WINDOWS
   tcr->shutdown_count = PTHREAD_DESTRUCTOR_ITERATIONS;
 #else
@@ -1362,9 +1373,11 @@ shutdown_thread_tcr(void *arg)
     if (tcr->flags & (1<<TCR_FLAG_BIT_FOREIGN)) {
       LispObj callback_macptr = nrs_FOREIGN_THREAD_CONTROL.vcell,
 	callback_ptr = ((macptr *)ptr_from_lispobj(untag(callback_macptr)))->address;
-    
+      int (*foreign_thread_control)(int) = (int (*)(int))callback_ptr;
+
       tsd_set(lisp_global(TCR_KEY), TCR_TO_TSD(tcr));
-      ((void (*)())ptr_from_lispobj(callback_ptr))(1);
+      foreign_thread_control(1);
+      //((void (*)(int))ptr_from_lispobj(callback_ptr))(1);
       tsd_set(lisp_global(TCR_KEY), NULL);
     }
 #ifdef DARWIN
@@ -1683,56 +1696,6 @@ lisp_thread_entry(void *param)
 #endif
 }
 
-typedef 
-short (*suspendf)();
-
-
-void
-suspend_current_cooperative_thread()
-{
-  static suspendf cooperative_suspend = NULL;
-  void *xFindSymbol(void*,char*);
-
-  if (cooperative_suspend == NULL) {
-    cooperative_suspend = (suspendf)xFindSymbol(NULL, "SetThreadState");
-  }
-  if (cooperative_suspend) {
-    cooperative_suspend(1 /* kCurrentThreadID */,
-                        1 /* kStoppedThreadState */,
-                        0 /* kAnyThreadID */);
-  }
-}
-
-void *
-cooperative_thread_startup(void *arg)
-{
-
-  TCR *tcr = get_tcr(0);
-  LispObj *start_vsp;
-
-  if (!tcr) {
-    return NULL;
-  }
-#ifndef WINDOWS
-  pthread_cleanup_push(tcr_cleanup,(void *)tcr);
-#endif
-  SET_TCR_FLAG(tcr,TCR_FLAG_BIT_AWAITING_PRESET);
-  start_vsp = tcr->save_vsp;
-  do {
-    SEM_RAISE(TCR_AUX(tcr)->reset_completion);
-    suspend_current_cooperative_thread();
-      
-    start_lisp(tcr, 0);
-    tcr->save_vsp = start_vsp;
-  } while (tcr->flags & (1<<TCR_FLAG_BIT_AWAITING_PRESET));
-#ifndef WINDOWS
-  pthread_cleanup_pop(true);
-#else
-  tcr_cleanup(tcr);
-#endif
-  return NULL;
-}
-
 void *
 xNewThread(natural control_stack_size,
 	   natural value_stack_size,
@@ -1764,6 +1727,11 @@ xNewThread(natural control_stack_size,
   return TCR_TO_TSD(activation.tcr);
 }
 
+void *
+do_nothing(void) {
+  return 0;
+}
+
 Boolean
 active_tcr_p(TCR *q)
 {
@@ -1886,6 +1854,7 @@ get_tcr(Boolean create)
   if ((current == NULL) && create) {
     LispObj callback_macptr = nrs_FOREIGN_THREAD_CONTROL.vcell,
       callback_ptr = ((macptr *)ptr_from_lispobj(untag(callback_macptr)))->address;
+    int (*foreign_thread_control)(int) = (int (*)(int))callback_ptr;
     int i, nbindwords = 0;
     extern natural initial_stack_size;
     
@@ -1916,13 +1885,13 @@ get_tcr(Boolean create)
       *(--current->save_vsp) = 0;
       current->vs_area->active -= node_size;
     }
-    nbindwords = ((int (*)())ptr_from_lispobj(callback_ptr))(-1);
+    nbindwords = foreign_thread_control(-1);
     for (i = 0; i < nbindwords; i++) {
       *(--current->save_vsp) = 0;
       current->vs_area->active -= node_size;
     }
     TCR_AUX(current)->shutdown_count = 1;
-    ((void (*)())ptr_from_lispobj(callback_ptr))(0);
+    foreign_thread_control(0);
 
   }
   
@@ -2086,7 +2055,7 @@ create_thread_context_frame(mach_port_t, natural *, siginfo_t *, TCR*, native_th
 
 Boolean mach_suspend_tcr(TCR *tcr)
 {
-  thread_act_t thread = (thread_act_t)(tcr->native_thread_id);
+  mach_port_t thread = (mach_port_t)((intptr_t)tcr->native_thread_id);
   kern_return_t kret = thread_suspend(thread);
 
   if (kret == 0) {
@@ -2248,7 +2217,7 @@ resume_tcr(TCR *tcr)
 Boolean mach_resume_tcr(TCR *tcr)
 {
   ExceptionInformation *xp = tcr->suspend_context;
-  mach_port_t thread = (mach_port_t)(tcr->native_thread_id);
+  mach_port_t thread = (mach_port_t)((intptr_t)(tcr->native_thread_id));
 #if WORD_SIZE == 64
   MCONTEXT_T mc = UC_MCONTEXT(xp);
 #else
