@@ -1,19 +1,18 @@
 ;;;-*- Mode: Lisp; Package: CCL -*-
 ;;;
-;;;   Copyright (C) 2009 Clozure Associates
-;;;   Copyright (C) 1994-2001 Digitool, Inc
-;;;   This file is part of Clozure CL.  
+;;; Copyright 1994-2009 Clozure Associates
 ;;;
-;;;   Clozure CL is licensed under the terms of the Lisp Lesser GNU Public
-;;;   License , known as the LLGPL and distributed with Clozure CL as the
-;;;   file "LICENSE".  The LLGPL consists of a preamble and the LGPL,
-;;;   which is distributed with Clozure CL as the file "LGPL".  Where these
-;;;   conflict, the preamble takes precedence.  
+;;; Licensed under the Apache License, Version 2.0 (the "License");
+;;; you may not use this file except in compliance with the License.
+;;; You may obtain a copy of the License at
 ;;;
-;;;   Clozure CL is referenced in the preamble as the "LIBRARY."
+;;;     http://www.apache.org/licenses/LICENSE-2.0
 ;;;
-;;;   The LLGPL is also available online at
-;;;   http://opensource.franz.com/preamble.html
+;;; Unless required by applicable law or agreed to in writing, software
+;;; distributed under the License is distributed on an "AS IS" BASIS,
+;;; WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+;;; See the License for the specific language governing permissions and
+;;; limitations under the License.
 (in-package "CCL")
 
 
@@ -1123,12 +1122,11 @@
           (xload-save-list (mapcar #'xload-save-list *xload-early-class-cells*)))
     (setf (xload-symbol-value (xload-copy-symbol '*istruct-cells*))
           *xload-early-istruct-cells*)
-    (let* ((svnrev (local-svn-revision))
-           (tree (svn-tree)))
+    (let ((rev (local-vc-revision)))
       (setf (xload-symbol-value (xload-copy-symbol '*openmcl-svn-revision*))
-            (typecase svnrev
-              (fixnum (ash svnrev *xload-target-fixnumshift*))
-              (string (xload-save-string (if tree (format nil "~a-~a" svnrev tree) svnrev)))
+            (typecase rev
+              (fixnum (ash rev *xload-target-fixnumshift*))
+              (string (xload-save-string rev))
               (t *xload-target-nil*))))
     (let* ((experimental-features *build-time-optional-features*))
       (setf (xload-symbol-value (xload-copy-symbol '*optional-features*))
